<?php

namespace App;


use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Validation\ValidationException;

class Tin
{
    /**
     * Get list of paginated users.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
//    public function getIndividual(Request $request)
//    {
//
//        $response = Http::withHeaders([
//            'cookie' => 'foo',
//            'X-Second' => 'bar'
//        ])->post('https://bornoirs.com/tin2/individual', [
//            'name' => 'peter pann',
//            'address' => 'no 27 kurum street, VI, Lagos',
//            'business' => 'Shoe Maker',
//            'mobile' => '09011223344',
//            'email' => 'bobo7@gmail.com',
//            'dob' => '1992-06-01 00:35:07',
//            'nin' => '6633772283',
//            'CID' => '3449923457-test',
//
//        ]);
//
//
//        return $response;
//    }

    /**
     * Get a user by ID.
     *
     * @param  int  $id
     * @return array
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException
     */
    public function getUserById(int $id): array
    {
        $user = User::findOrFail($id);

        return fractal($user, new UserTransformer())->toArray();
    }

    /**
     * Store a new user.
     *
     * @param  array  $attrs
     * @return array
     *
     * @throws \Illuminate\Validation\ValidationException
     */
    public function storeUser(array $attrs): array
    {
        $user = new User($attrs);

        if (!$user->isValidFor('CREATE')) {
            throw new ValidationException($user->validator());
        }

        $user->save();

        event(new UserCreated($user));

        return fractal($user, new UserTransformer())->toArray();
    }

    /**
     * Update a user by ID.
     *
     * @param  int  $id
     * @param  array  $attrs
     * @return array
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException
     * @throws \Illuminate\Validation\ValidationException
     */
    public function updateUserById(int $id, array $attrs): array
    {
        $user = User::findOrFail($id);
        $user->fill($attrs);

        if (!$user->isValidFor('UPDATE')) {
            throw new ValidationException($user->validator());
        }

        $changes = $user->getDirty();
        $user->save();

        event(new UserUpdated($user, $changes));

        return fractal($user, new UserTransformer())->toArray();
    }

    /**
     * Delete a user by ID.
     *
     * @param  int  $id
     * @return bool
     *
     * @throws \Illuminate\Database\Eloquent\ModelNotFoundException
     */
    public function deleteUserById(int $id): bool
    {
        $user = User::findOrFail($id);

        return (bool) $user->delete();
    }
}
